//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//DX.H - Header file for the CDX class
//Copyright Marshall Harnish /97
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#ifndef _CDX_INC_
#define _CDX_INC_
//#define _FULL_SCREEN_	//Comment this out to run in windowed mode! (VERY useful for debugging!)

#include <afxwin.h>	//MFC include
#include "ddraw.h"	//DirectDraw include
#include "globals.h"//Globals include

class CDX
{
public:
	CDX();	//Constructor
	~CDX();	//Destructor

	//* CDX Creation functions
	int Create(int Width,int Height,int BPP);	//Call this to create the CDX object
	int InitDD(int Width,int Height,int BPP);	//Initializes DirectDraw and sets the video mode
	int InitSurfaces(void);		//Initializes the DD surfaces (primary, back, tile, etc)

	//* CDX Graphics manipulation primatives
	int LoadBitmap(char *iFileName,int Surface,int DestX,int DestY);	//Loads a bitmap and displays it on the specified surface
	int FlipSurfaces(void);				//Flips the surfaces
	int CopyArea(CRect *SourceRect,CPoint *DestPt,int SourceSurface,int DestSurface);	//Copys a section of one surface to another
	int FillArea(CRect *DestRect,DWORD Color,int Surface);		//Fills an area on a surface with the specified color
	int ResizeTS(int x, int y);

	//* CDX Graphics manipulation advanced functions
	int FillTexturedPoly(int x1,int y1,int x2,int y2,int x3,int y3,int x4,int y4,int TileNum, int SrcSurface);
private:
protected:
	LPDIRECTDRAW m_pDD;				//The primary DirectDraw object
	LPDIRECTDRAWSURFACE m_Primary;	//The PRIMARY surface (the one the user sees)
	LPDIRECTDRAWSURFACE m_Back;		//The BACK surface (is part of a complex flipping structure with the PRIMARY surface)
	LPDIRECTDRAWSURFACE m_Tiles;	//The TILES surface (is used to store tile info, etc.  Not part of the flipping structure)
	LPDIRECTDRAWSURFACE m_Work;		//A temporary surface used for misc. texturemapping routines
	
	//Information regarding the edges of a polygon (used for the texture mapping function	
	struct PolyEdgeScan
	{
		int x;
		int TextureX;
		int TextureY;
	};
	PolyEdgeScan *fwdScan;	//Holds info about the scanlines
	PolyEdgeScan *bwdScan;
	CPoint TexturePoint[4];	//Holds the points of the source texture map to be used (will always be the 32x32 tile)... declared here for speed considerations (so that it doesnt have to initialize it everytime

	//* Internal CDX functions (shouldn't be called directly, unless from a member fn)
	LPSTR LockSurfaceMem(int Surface);	//Locks a surface and returns a pointer to the memory
	void UnlockSurfaceMem(int Surface,LPSTR SurfaceMem);	//Unlocks a surface
	void TraceErrorDD(HRESULT hErr);	//Displays a DirectDraw error (using TRACE)
	void ScanEdge(PolyEdgeScan *Edge,CPoint *DestA,CPoint *DestB,CPoint *TextureA,CPoint *TextureB);	//Scans a polygon edge (used by FillTexturedPoly)
	void DrawTexturedHLine(LPSTR m_DestBits,PolyEdgeScan *Edge1,PolyEdgeScan *Edge2,LPSTR m_SrcBits);	//Draws a texture mapped horizontal line (used by FillTexturedPoly)
};

#endif